/*
© ttcoder 2018
A little HDA verbs decoder for debugging
https://dev.haiku-os.org/attachment/ticket/14242/hda_trace_reconfig.txt


Can be used in-line within the hda driver, for better dprintf() output;
Can be made into a stand-alone terminal command (with #define STANDALONE_CLI_TOOL):

	gcc -I/system/develop/headers/private/media/ _hda_verb_dump.cpp
*/

//#define STANDALONE_CLI_TOOL



#ifdef STANDALONE_CLI_TOOL
#	include <SupportDefs.h>
#	include <stdio.h>
#	include "hda_codec_defs.h"
#	undef TRACE
#	define TRACE(a...) printf("hda: " a)
#endif



void Hda_Dump( const uint32 verb )
{
	uint32 v = verb;
	
	int payload_ = 0;  // set below
	v >>= 8;
	const int vid = v & 0xfff;  // +"potential_vid" !
	v >>= 12;
	const int nid = v & 0xff;
	v >>= 8;
	const int codec = v;
	
#if 0
	if( codec != 0 )
	{
		TRACE( "      hda-verb to unrelated codec [%d]..\n", codec );
		
		//
		return;
		//
	}
#endif

	int extracted_verb = 0;
	const char * pretty = "?";
	{
		const int haiku_parlance_verb = vid << 8;  // our (haiku) defines go like e.g. "0xf0100" instead of "0xf01"
		
		/*
		HDA spec §7.3:
			There are two types of verbs: those with 4-bit identifiers and 16 bits
			of data payload and those with 12-bit identifiers and 8 bits of payload.
		*/
		
		const int verb_4bit  = haiku_parlance_verb & 0xf0000;
		const int verb_12bit = haiku_parlance_verb & 0xfff00;
		
		bool is_4bit_verb = false;
		
		//+ if verb_4bit != 0x07.. && verb_4bit != 0x0F..then:
		switch( verb_4bit )
		{
			/*
			Those with e.g. RealTek chips might want to also trace those:
			
			VID_GET_COEFFICIENT_INDEX:  // 0xd0000
			VID_SET_COEFFICIENT_INDEX:  // 0x50000
			VID_GET_PROCESSING_COEFFICIENT:  // 0xc0000
			VID_SET_PROCESSING_COEFFICIENT:  // 0x40000
			*/
			
			case VID_GET_AMPLIFIER_GAIN_MUTE:  // 0xb0000
				is_4bit_verb = true;
				pretty = "VID_GET_AMPLIFIER_GAIN_MUTE";
			break;
			
			case VID_SET_AMPLIFIER_GAIN_MUTE:  // 0x30000
				is_4bit_verb = true;
				pretty = "VID_SET_AMPLIFIER_GAIN_MUTE";
			break;
			
			case VID_GET_CONVERTER_FORMAT:  // 0xa0000
				is_4bit_verb = true;
				pretty = "VID_GET_CONVERTER_FORMAT";
			break;
			
			case VID_SET_CONVERTER_FORMAT:  // 0x20000
				is_4bit_verb = true;
				pretty = "VID_SET_CONVERTER_FORMAT";
			break;
			
			// case 0x70000
			// case 0xF0000
			// => those are 16bit verbs, not 4-bit ones
		}
		
		if( false == is_4bit_verb )
		{
			switch( verb_12bit )
			{
				case VID_GET_CONFIGURATION_DEFAULT:  // 0xf1c00
					pretty = "VID_GET_CONFIGURATION_DEFAULT";
				break;
				
				case VID_GET_PARAMETER:  // 0xf0000
					pretty = "VID_GET_PARAMETER";
				break;
				
				case VID_GET_CONNECTION_LIST_ENTRY:  // 0xf0200
					pretty = "VID_GET_CONNECTION_LIST_ENTRY";
				break;
				
				case VID_GET_CONVERTER_STREAM_CHANNEL:  // 0xf0600
					pretty = "VID_GET_CONVERTER_STREAM_CHANNEL";
				break;
				
				case VID_GET_PIN_WIDGET_CONTROL:  // 0xf0700
					pretty = "VID_GET_PIN_WIDGET_CONTROL";
				break;
				
				case VID_GET_PINSENSE:  // 0xF0900
					pretty = "VID_GET_PINSENSE";
				break;
				
				case VID_GET_EAPDBTL_EN:  // 0xF0C00
					pretty = "VID_GET_EAPDBTL_EN";
				break;
				
				case VID_SET_CONNECTION_SELECT:  // 0x70100
					pretty = "VID_SET_CONNECTION_SELECT";
				break;
				
				case VID_SET_POWER_STATE:  // 0x70500
					pretty = "VID_SET_POWER_STATE";
				break;
				
				case VID_SET_CONVERTER_STREAM_CHANNEL:  // 0x70600
					pretty = "VID_SET_CONVERTER_STREAM_CHANNEL";
				break;
				
				case VID_SET_PIN_WIDGET_CONTROL:  // 0x70700
					pretty = "VID_SET_PIN_WIDGET_CONTROL";
				break;
				
				case VID_SET_UNSOLRESP:  // 0x70800
					pretty = "VID_SET_UNSOLRESP";
				break;
				
				case VID_SET_EAPDBTL_EN:  // 0x70C00
					pretty = "VID_SET_EAPDBTL_EN";
				break;

#if 1
				// extra verbs used by codec 3 (HDMI)..
				
				case VID_SET_DIGITAL_CONVERTER_CONTROL1:  // 0x70d00
					pretty = "VID_SET_DIGITAL_CONVERTER_CONTROL1";
				break;
				
				case VID_SET_DIGITAL_CONVERTER_CONTROL2:  // 0x70e00
					pretty = "VID_SET_DIGITAL_CONVERTER_CONTROL2";
				break;
				
				case VID_SET_CONVERTER_CHANNEL_COUNT:  // 0x72d00
					pretty = "VID_SET_CONVERTER_CHANNEL_COUNT";
				break;
				
				case VID_SET_DATA_ISLAND_PACKET_INDEX:  // 0x73000
					pretty = "VID_SET_DATA_ISLAND_PACKET_INDEX";
				break;
				
				case VID_SET_DATA_ISLAND_PACKET_DATA:  // 0x73100
					pretty = "VID_SET_DATA_ISLAND_PACKET_DATA";
				break;
				
				case VID_SET_DATA_ISLAND_PACKET_XMITCTRL:  // 0x73200
					pretty = "VID_SET_DATA_ISLAND_PACKET_XMITCTRL";
				break;
				
				case VID_SET_ASP_CHANNEL_MAPPING:  // 0x73400
					pretty = "VID_SET_ASP_CHANNEL_MAPPING";
				break;
				
				case VID_GET_CONVERTER_CHANNEL_COUNT:  // 0xf2d00
					pretty = "VID_GET_CONVERTER_CHANNEL_COUNT";
				break;
				
				case VID_GET_DATA_ISLAND_PACKET_DATA:  // 0xf3100
					pretty = "VID_GET_DATA_ISLAND_PACKET_DATA";
				break;
				
				case VID_GET_DATA_ISLAND_PACKET_XMITCTRL:  // 0xf3200
					pretty = "VID_GET_DATA_ISLAND_PACKET_XMITCTRL";
				break;
#endif
			}
		}
		
		if( is_4bit_verb )
		{
			payload_ = verb & 0xffff;
			
			// haiku-parlance -> spec parlance:
			extracted_verb = verb_4bit >> 16;
		}
		else
		{
			payload_ = verb & 0xff;
			
			// haiku-parlance -> spec parlance:
			extracted_verb = verb_12bit >> 8;
		}
	}
	
	TRACE( "  hda-verb       <0x%8.8lx>:  [%d] nid 0x%x (%d) :  vid 0x%x (%s)   pload 0x%x\n"
		,verb, codec
		,nid, nid
		,extracted_verb ,pretty
		,payload_
		);
}




#ifdef STANDALONE_CLI_TOOL

int main( int argc, char ** argv )
{
	if( argc > 2 )
	{
		printf(" ** need 0 or 1 args (got %d)\n", argc );
		
		return 1;
	}
	
	if( argc == 2 )
	{
		uint32 hex_deco = 0;
		sscanf( argv[1], "%lx", &hex_deco );
		
		Hda_Dump( hex_deco );
		
		return 0;
	}
	// else => argc==1, no arg,  use STDIN:
	
	char buf[256] = {};
	;
	do
	{
		buf[0] = '\0';
		
		/*const char * c = */gets( buf );
		
		//printf(".. <%s> <%s>\n", c, buf);
		uint32 hex_deco = 0;
		sscanf( buf, "%lx", &hex_deco );
		Hda_Dump( hex_deco );
	}
	while( strlen(buf) );
	
	return 0;
}

#endif




